#include "vamp-hostsdk/PluginHostAdapter.h"

#include "vamp-hostsdk/Files.h"

#include <cctype> // tolower
#include <cstring>
#include <cstdlib>
#include <dirent.h>
#include <dlfcn.h>

#define PLUGIN_SUFFIX "so"


std::vector<std::string> Files::listLibraryFiles() 
{
  return listLibraryFilesMatching(Filter());
}

std::vector<std::string> Files::listLibraryFilesMatching(Filter filter)
{
  std::vector<std::string> path = Vamp::PluginHostAdapter::getPluginPath();
  std::vector<std::string> libraryFiles;

    // we match case-insensitively, but only with ascii range
    // characters (input strings are expected to be utf-8)
  std::vector<std::string> libraryNames;
    for (int j = 0; j < int(filter.libraryNames.size()); ++j) {
      std::string n = filter.libraryNames[j];
        for (size_t i = 0; i < n.length(); ++i) {
            if (!(n[i] & 0x80)) {
                n[i] = char(tolower(n[i]));
            }
        }
        libraryNames.push_back(n);
    }

    for (size_t i = 0; i < path.size(); ++i) {
        
      std::vector<std::string> files = listFiles(path[i], PLUGIN_SUFFIX);

        for (std::vector<std::string>::iterator fi = files.begin();
             fi != files.end(); ++fi) {

            // we match case-insensitively, but only with ascii range
            // characters (this string is expected to be utf-8)
          std::string cleaned = *fi;
            for (size_t j = 0; j < cleaned.length(); ++j) {
                if (!(cleaned[j] & 0x80)) {
                    cleaned[j] = char(tolower(cleaned[j]));
                }
            }

            // libraryName should be lacking an extension, as it is
            // supposed to have come from the plugin key
            std::string::size_type pi = cleaned.find('.');
            if (pi != std::string::npos) {
                cleaned = cleaned.substr(0, pi);
            }
            
            bool matched = false;

            switch (filter.type) {

            case Filter::All:
                matched = true;
                break;

            case Filter::Matching:
                for (int j = 0; j < int(libraryNames.size()); ++j) {
                    if (cleaned == libraryNames[j]) {
                        matched = true;
                        break;
                    }
                }
                break;

            case Filter::NotMatching:
                matched = true;
                for (int j = 0; j < int(libraryNames.size()); ++j) {
                    if (cleaned == libraryNames[j]) {
                        matched = false;
                        break;
                    }
                }
                break;
            }

            if (!matched) continue;
            
            std::string fullPath = path[i];
            fullPath = splicePath(fullPath, *fi);
	    libraryFiles.push_back(fullPath);
	}
    }

  return libraryFiles;
}

void* Files::loadLibrary(std::string path)
{
  void *handle = 0;
  handle = dlopen(path.c_str(), RTLD_LAZY | RTLD_LOCAL);
  if (!handle) {
    std::cerr << "Vamp::HostExt: Unable to load library \""
           << path << "\": " << dlerror() << std::endl;
  }
  return handle;
}

void Files::unloadLibrary(void *handle)
{
  dlclose(handle);
}

void* Files::lookupInLibrary(void *handle, const char *symbol)
{
  return (void *)dlsym(handle, symbol);
}

std::string Files::lcBasename(std::string path)
{
  std::string basename(path);
	
  std::string::size_type li = basename.rfind('/');
  if (li != std::string::npos) basename = basename.substr(li + 1);

  li = basename.find('.');
  if (li != std::string::npos) basename = basename.substr(0, li);

  // case-insensitive, but only with ascii range characters (this
  // string is expected to be utf-8)
  for (size_t i = 0; i < basename.length(); ++i) {
      if (!(basename[i] & 0x80)) {
          basename[i] = char(tolower(basename[i]));
      }
  }

  return basename;
}

std::string Files::splicePath(std::string a, std::string b)
{
  return a + "/" + b;
}

std::vector<std::string> Files::listFiles(std::string dir, std::string extension)
{
  std::vector<std::string> files;

  size_t extlen = extension.length();
  DIR* d = opendir(dir.c_str());
  if (!d) return files;
          
  struct dirent* e = 0;

  while ((e = readdir(d))) 
  {

    size_t len = strlen(e->d_name);
    if (len < extlen + 2 || e->d_name + len - extlen - 1 != "." + extension)
      continue;

    files.push_back(e->d_name);
  }

  closedir(d);

  return files;
}

bool Files::isNonNative32Bit()
{
  // Return true if we are running on a system for which we should
  // use the VAMP_PATH_32 variable instead of VAMP_PATH. This will
  // be the case if we are a 32-bit executable but the OS is
  // natively 64-bit.
  //
  // This currently works only on Windows; other operating systems
  // will use VAMP_PATH always.
  if (sizeof(void *) == 8) 
    return false;
  return false;
}

bool Files::getEnvUtf8(std::string variable, std::string &value)
{
  value = "";
  char *val = getenv(variable.c_str());
  if (!val) {
      return false;
  }

  value = val;
  return true;
}
