#pragma once

#include "vamp-sdk/Plugin.h"
#include "vamp.h"
#include <vector>

namespace Vamp {

/**
 * \class PluginHostAdapter PluginHostAdapter.h <vamp-hostsdk/PluginHostAdapter.h>
 * 
 * PluginHostAdapter is a wrapper class that a Vamp host can use to
 * make the C-language VampPluginDescriptor object appear as a C++
 * Vamp::Plugin object.
 *
 * The Vamp API is defined in vamp/vamp.h as a C API.  The C++ objects
 * used for convenience by plugins and hosts actually communicate
 * using the C low-level API, but the details of this communication
 * are handled seamlessly by the Vamp SDK implementation provided the
 * plugin and host use the proper C++ wrapper objects.
 *
 * See also PluginAdapter, the plugin-side wrapper that makes a C++
 * plugin object available using the C query API.
 */

class PluginHostAdapter : public Plugin
{
public:
    PluginHostAdapter(const VampPluginDescriptor *descriptor,
                      float inputSampleRate);
    virtual ~PluginHostAdapter();
    
    static std::vector<std::string> getPluginPath();

    bool initialise(size_t channels, size_t stepSize, size_t blockSize);
    void reset();

    InputDomain getInputDomain() const;

    unsigned int getVampApiVersion() const;
    std::string getIdentifier() const;
    std::string getName() const;
    std::string getDescription() const;
    std::string getMaker() const;
    int getPluginVersion() const;
    std::string getCopyright() const;

    ParameterList getParameterDescriptors() const;
    float getParameter(std::string) const;
    void setParameter(std::string, float);

    ProgramList getPrograms() const;
    std::string getCurrentProgram() const;
    void selectProgram(std::string);

    size_t getPreferredStepSize() const;
    size_t getPreferredBlockSize() const;

    size_t getMinChannelCount() const;
    size_t getMaxChannelCount() const;

    OutputList getOutputDescriptors() const;

    FeatureSet process(const float *const *inputBuffers, RealTime timestamp);

    FeatureSet getRemainingFeatures();

protected:
    void convertFeatures(VampFeatureList *, FeatureSet &);

    const VampPluginDescriptor *m_descriptor;
    VampPluginHandle m_handle;
};

}
