#include "vamp-hostsdk/PluginWrapper.h"

namespace Vamp {

namespace HostExt {

PluginWrapper::PluginWrapper(Plugin *plugin) :
    Plugin(plugin->getInputSampleRate()),
    m_plugin(plugin)
{
}

PluginWrapper::~PluginWrapper()
{
    delete m_plugin;
}

bool
PluginWrapper::initialise(size_t channels, size_t stepSize, size_t blockSize)
{
    return m_plugin->initialise(channels, stepSize, blockSize);
}

void
PluginWrapper::reset()
{
    m_plugin->reset();
}

Plugin::InputDomain
PluginWrapper::getInputDomain() const
{
    return m_plugin->getInputDomain();
}

unsigned int
PluginWrapper::getVampApiVersion() const
{
    return m_plugin->getVampApiVersion();
}

std::string
PluginWrapper::getIdentifier() const
{
    return m_plugin->getIdentifier();
}

std::string
PluginWrapper::getName() const
{
    return m_plugin->getName();
}

std::string
PluginWrapper::getDescription() const
{
    return m_plugin->getDescription();
}

std::string
PluginWrapper::getMaker() const
{
    return m_plugin->getMaker();
}

int
PluginWrapper::getPluginVersion() const
{
    return m_plugin->getPluginVersion();
}

std::string
PluginWrapper::getCopyright() const
{
    return m_plugin->getCopyright();
}

PluginBase::ParameterList
PluginWrapper::getParameterDescriptors() const
{
    return m_plugin->getParameterDescriptors();
}

float
PluginWrapper::getParameter(std::string parameter) const
{
    return m_plugin->getParameter(parameter);
}

void
PluginWrapper::setParameter(std::string parameter, float value)
{
    m_plugin->setParameter(parameter, value);
}

PluginBase::ProgramList
PluginWrapper::getPrograms() const
{
    return m_plugin->getPrograms();
}

std::string
PluginWrapper::getCurrentProgram() const
{
    return m_plugin->getCurrentProgram();
}

void
PluginWrapper::selectProgram(std::string program)
{
    m_plugin->selectProgram(program);
}

size_t
PluginWrapper::getPreferredStepSize() const
{
    return m_plugin->getPreferredStepSize();
}

size_t
PluginWrapper::getPreferredBlockSize() const
{
    return m_plugin->getPreferredBlockSize();
}

size_t
PluginWrapper::getMinChannelCount() const
{
    return m_plugin->getMinChannelCount();
}

size_t PluginWrapper::getMaxChannelCount() const
{
    return m_plugin->getMaxChannelCount();
}

Plugin::OutputList
PluginWrapper::getOutputDescriptors() const
{
    return m_plugin->getOutputDescriptors();
}

Plugin::FeatureSet
PluginWrapper::process(const float *const *inputBuffers, RealTime timestamp)
{
    return m_plugin->process(inputBuffers, timestamp);
}

Plugin::FeatureSet
PluginWrapper::getRemainingFeatures()
{
    return m_plugin->getRemainingFeatures();
}

}

}
