#pragma once

#include "vamp.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct vhLibrary_t *vhLibrary;

/**
 * Return the number of Vamp plugin libraries discovered in the
 * installation path. This number will remain fixed after the first
 * call -- plugins are only discovered once, the first time this
 * function is called.
 */
extern int vhGetLibraryCount();

/**
 * Return the library name (base soname) of the library with the given
 * index, in the range 0..(vhGetLibraryCount()-1).
 */
extern const char *vhGetLibraryName(int library);

/**
 * Return the library index for the given library name, or -1 if the
 * name is not known.
 */
extern int vhGetLibraryIndex(const char *name);
    
/**
 * Load the library with the given index. If the library cannot be
 * loaded for any reason, the return value is 0; otherwise it is an
 * opaque pointer suitable for passing to other functions in this API.
 */
extern vhLibrary vhLoadLibrary(int library);

/**
 * Return the number of Vamp plugins in the given library.
 */
extern int vhGetPluginCount(vhLibrary library);

/**
 * Return a Vamp plugin descriptor for a plugin in a given
 * library. This simply calls the vampGetPluginDescriptor function in
 * that library with the given plugin index and returns the
 * result. See vamp/vamp.h for details about the plugin descriptor.
 */ 
extern const VampPluginDescriptor *vhGetPluginDescriptor(vhLibrary library,
							 int plugin);

/**
 * Unload a plugin library. Do not do this while any of its plugins
 * are still in use.
 */
extern void vhUnloadLibrary(vhLibrary);
    
#ifdef __cplusplus
}
#endif
